%%  Growing a Compiler
%
% by Bill McKeeman and Lu He
%
% MathWorks and Dartmouth, May 2009
%% Abstract
% Self-compiling compilers are common.  
% The question is:  How far can one go, bootstrapping a (very) 
% small compiler-compiler into more capable compilers?
%
% Context-free grammars are extended to accomodate output.  
% A grammar executing machine (GEM) is introduced which accepts an 
% input text and a grammar, and outputs another text.  
% Both the input text and the output text can also be grammars, 
% permitting the production of ever more powerful grammars.  
% GEM itself can be extended to build-in the capabilities of the 
% previous grammars.  
% The rules of the game require that changing GEM does not add
% to its original capability -- 
% it merely makes the implementation more robust or faster.
%
% The grammars and the machine have some simple symmetries that lead to
% actions such as backtracking and decompiling.  It is also possible to
% directly execute bit-strings in the Intel x86 hardware.
%% Chapters
% 
% <html>
% <ol>
% <li><a href="GrowingCompilerCh1.html">Base GEM</a>
%   <ul>
%    <li>statement of the problem</li>
%    <li>executable grammars</li>
%    <li>simple examples</li>
%   </ul>
% </li>
% <li><a href="GrowingCompilerCh2.html">Robust GEM</a>
%   <ul>
%     <li>pre-entered character classes</li>
%     <li>using nowhite, pretty, invert</li>
%   </ul>
% </li>
% <li><a href="GrowingCompilerCh3.html">GEM with builtin nowhite and chars</a>
%   <ul>
%     <li>using multi-character input and output symbols</li>
%     <li>left-associative arithmetic expressions</li>
%     <li>X86 floating point stack</li>
%   </ul>
% </li>
% <li><a href="GrowingCompilerCh4.html">GEM with builtin multichar symbols</a>
%   <ul>
%     <li>using Kleene * and <sup>+</sup> in executable grammars</li>
%   </ul>
% </li>
% <li><a href="GrowingCompilerCh5.html">Running Intel X86 code</a>
%    <ul>
%      <li>X86 Assembler</li>
%      <li>calculator</li>
%      <li>atoi</li>
%    </ul>
% </li>
% <li><a href="GrowingCompilerCh6.html">Multichar Phrase Names</a>
%    <ul>
%      <li>better treatment of Kleene * and +</li>
%      <li>new nowhite, pretty, invert</li>
%    </ul>
% </li>
% </ol>
% </html>
% 
%% Notes
% The origin of the idea is a undergraduate thesis (UC Santa Cruz, 1978)
% written by Doug Michels under the supervsion of Bill McKeeman.
%
% The title is inspired by:
% Guy Steele's 1998 OOPSLA talk _Growing a Language_.
%
% Thanks to Steve Johnson and Alan Martin for critical advice in the 
% preparation of this presentation.
%
% The default font sizes in Firefox are uncomfortably large for this paper.
% Try [view][edit][zoom][text only][zoom out][zoom out].
%
%% References
% * Bill McKeeman <http://www.cs.dartmouth.edu/~mckeeman/downloads.html
%   Compiler and Compiler Course>
% * Wikipedia <http://en.wikipedia.org/wiki/Mealy_machine Mealy Machine>
% * Guy Steele <http://www.brics.dk/~hosc/local/HOSC-12-3-pp221-236.pdf
%   Growing a Language>. 
% * Doug Michels <michels1978.pdf
%   A concise extensible metalanguage for translator implementation>

%% Signatures
%
% * _Bill McKeeman_ , MathWorks Fellow
% * _Lu He_, Computer Science Department, Dartmouth
%
% An earlier version was presented to the Computer Science Colloquium, 
% Stanford, March 4, 2009

