% FILE:        gem2.m
% PURPOSE:     Machine to execute self-describing grammars
%              Built-in de-whiting of the input IOG
% EXAMPLE:     
%   obj = gem2();                          % constructor
%   o = obj.run('x y z', gram)             % run gram on input
%   o = obj.run('x y', gram, 'T')          % .. and trace steps
% REQUIRES:
%   iog2.c iog2.h
% TEST:
%   testGem2()

% COPYRIGHT:   1988 W. M. McKeeman.  See license.txt.
% MODS:	    1988.06.02 -- mckeeman -- original
%           2008.06.11 -- mckeeman@dartmouth -- moved to MEX
%           2009.04.09 -- luhe@dartmouth     -- add classes; remove nowhite

function obj = gem2()
  
  EOL = char(10);                                % ASCII end-of-line

  asciiIOG = sprintf('A=''%c''"%c";', repeat(char(1:126), 2));

  nowhite = [        % Requires asciiIOG
    'g = p g;'          ...   list of p-s
    'g =;'              ...
    'p ='' '';'         ...   discard blank
    'p =''' EOL ''';'   ...   discard EOL
    'p = I A I;'        ...   keep input symbols
    'p = O A O;'        ...   keep output symbols
    'p = A;'            ...   keep everything else
    'I = ''''''"''";'   ...   see ', send '
    'O = ''"''""";'     ...   see ", send "
    asciiIOG
  ];

  obj = public();
  %return;
  
  % ----------------------------------------------------
  
  % Run GEM.  gtxt must be deblanked.
  function otxt = run(itxt, gtxt, flag)
    if nargin == 2, flag = ''; end
    otxt = iog2(itxt, gtxt, flag);
  end

  function o = public()
    o         = struct;
    o.nowhite = nowhite;
    o.run     = @run;  % execute GEM
  end

end
