/* Grammar Executing Machine version 0 -- simple and risky */
static void
gem(void) {
  for (;;) {        
    switch (mode) {
      /* executing rules */
      case PARSE:
        switch (*p) {
        ALPHA                             /* call new rule */
          ps++; *ps = p;                    /*   save return address */
          p = p0; mode = SEARCH; break;     /*   search from beginning */
        case '\'':                        /* input */
          p++;                              /*   skip over 1rst ' */
          if (*p == *i) {i++; p++; p++;}    /*   read-match */
          else {mode = BACK; p--; p--;}     /*   read-mismatch */
          break;
        case '"':                         /* output */
          p++; o++;                         /*   skip over 1rst " */
          *o = *p;                          /*   move literal to output*/
          p++; p++; break;                  /*   skip over 2nd " */
        case ';':                         /* rule end (parsing) */
          p--;                              /*   back up over ; */
          bs++; *bs = p;                    /*   save backup pointer */
          if (psize < 0) return;            /*   empty stack: success */
          p = *ps; ps--;                    /*   return from rule */
          p++; break;                       /*   skip over rule name */
        default:                          /* bad char in grammar */
          error("Unexpected character (PARSE)");
        }
        break;                            /* end of parse step */
        
      /* backtracking */
      case BACK:
      switch (*p) {
        ALPHA                             /* un-return from rule */
          ps++; *ps = p;                    /* save return address */
          p = *bs; bs--; break;             /* end of previous rule */
        case '\'':                        /* input */
          i--;                              /* un-get input */
          p--; p--; p--; break;             /* un-skip literal */
        case '"':                         /* output */
          o--;                              /* un-put output */
          p--; p--; p--; break;             /* un-skip literal */
        case '=':                         /* rule begin (backtracking) */
          mode = SEARCH;                    /* forward again */
          p++; break;                       /* skip by = */
        default:
          error("Unexpected character (BACK)");
        }
        break;                            /* end of back step */
        
      /* searching for a rule */
      case SEARCH:
        switch (*p) {
        ALPHA                             /* phrase name */
          p++; break;                       /* skip over name */
        case '\'': 
        case '"':                         /* input/output */
          p++; p++; p++; break;             /* skip over 'x' or "x" */
        case ';':                         /* rule coming */
          p++;                              /* skip over ; */
          if (p-p0==pN) {                   /* end of code */
            if (psize == 0) error("Unparsable input");
            p = *ps; ps--;                /* back out one rule */
            mode = BACK;                    /* reverse direction */
            p--; break;                     /* un-skip over name */
          }
          if (*p==**ps) mode = PARSE;       /* lhs is phrase name */
          p++; p++; break;                  /* skip over lhs = */
        default:
          error("Unexpected character (SEARCH)");
        }
        break;
    }
  }
}
