/* Grammar Executing Machine version 1 -- self-checking to avoid SEGVs */
static void
gem(void) {
  for (;;) {  
    if (traceit) TRACE();                 /* iog1(i,g,'T') */
    CHECKCODE();                          /* protect *p below */
    switch (mode) {
      /* executing rules */
      case PARSE:
        switch (*p) {    
        ALPHA                             /* call new rule */
          ps++; 
          CHECKPARSE();
          *ps = p;                          /*   save return address */
          p = p0; mode = SEARCH; break;     /*   search from beginning */
        case '\'':                        /* input */
          p++;                              /*   skip over 1rst ' */         
          CHECKCODE();                      /*   bounds check */
          if (*p == *i) {                   /*   read-match */
            i++; p++;                       /*   succeed, continue */
            CHECKCODE(); 
            if (*p!='\'') error("missing ' (PARSE)");    
            p++;
          } else {                          /*   read-mismatch */
            mode = BACK; p--; p--;          /*   fail, backtrack */
          }   
          break;
        case '"':                         /* output */
          p++; o++;                         /*   skip over 1rst " */
          CHECKCODE();                      /*   bounds check */
          CHECKOUT();                       /*   bounds check */
          *o = *p;                          /*   move literal to output*/
          p++; 
          CHECKCODE();                      /* bounds check */
          if (*p!='"') error("missing \" in (PARSE)");    
          p++; break;                       /*   skip over 2nd " */
        case ';':                         /* rule end (parsing) */
          p--;                              /*   back up over ; */
          bs++; 
          CHECKBACK();
          *bs = p;                          /*   save backup pointer */         
          if (psize < 0) return;            /*   empty stack: success */
          p = *ps; ps--;                    /*   return from rule */
          p++; break;                       /*   skip over rule name */
          
        default:                          /* bad char in grammar */
          error("Unexpected character (PARSE)");
        }
        break;                            /* end of parse step */
        
      /* backtracking */
      case BACK:
      switch (*p) {
        ALPHA                             /* un-return from rule */
          ps++; 
          CHECKPARSE();
          *ps = p;                          /* save return address */
          p = *bs; bs--; break;             /* end of previous rule */
        case '\'':                        /* input */
          i--;                              /* un-get input */
          p--; p--; p--; break;             /* un-skip literal */
        case '"':                         /* output */
          o--;                              /* un-put output */
          p--; p--; p--; break;             /* un-skip literal */
        case '=':                         /* rule begin (backtracking) */
          mode = SEARCH;                    /* forward again */
          p++; break;                       /* skip by = */
        default:
          error("Unexpected character (BACK)");
        }
        break;                            /* end of back step */
        
      /* searching for a rule */
      case SEARCH:
        switch (*p) {
        ALPHA                             /* phrase name */
          p++; break;                       /* skip over name */
        case '\'':                        /* input*/    
          p++; p++;
          CHECKCODE();                      /* bounds check */
          if (*p!='\'') error("missing ' (SEARCH)");    
          p++; break;                       /* skip over 'x' */
        case '"':                         /* output*/  
          p++; p++; 
          CHECKCODE();                      /* bounds check */
          if (*p!='"') error("missing \" (SEARCH)");   
          p++; break;                       /* skip over "x" */
        case ';':                         /* rule coming */
          p++;                              /* skip over ; */
          if (p-p0==pN) {                   /* end of code */
            if (psize == 0) error("unparsable input");
            p = *ps; ps--;                  /* back out one rule */
            mode = BACK;                    /* reverse direction */
            p--; break;                     /* un-skip over name */ 
          } 
          CHECKNAME(*p);                    /* rule name must be letter */
          if (*p==**ps) mode = PARSE;       /* lhs is phrase name */
          p++;                              /* skip over lhs */
          CHECKCODE();                      /* bounds check */
          if (*p!='=') error("missing = (SEARCH)");    
          p++; break;                       /* skip over = */
        default:
          error("Unexpected character (SEARCH)");
        }
        break;
    }
  }
}
