% FILE:     testGem0.m 
% PURPOSE:  test gem (the grammar executing machine)
% METHOD:   test a few grammars -- avoid bad inputs
% REQUIRES: gem0.m iog0.c iog0.h
% EXAMPLE:    
%   testGem0()

% COPYRIGHT: 2009 W. M. McKeeman.  See license.txt.

function testGem0()  
  tests  = 0;                                     % none yet
  passed = 0;                                     % none yet
  
  G = gem0();                                     % instantiate GEM object
  %       -------------- simple tests --------------------
  
  g = 'g=;';                                      % an empty grammar
  o = G.run('', g);                               % does not blow up
  positiveTest(isempty(o));
  
  g = 'g=''x''"y";';                              % accept 'x', emit 'y'
  o = G.run('x', g);                              % result y
  positiveTest(strcmp(o, 'y'));
  
  g = 'r=s;s=''1'';s=''2'';';                     % a '1' or a '2'
  o = G.run('1', g);                              % no output
  positiveTest(isempty(o));

  %       -------------- xxxCFG and xxxIOG tests ------------
  
  % test digits   (defines phrase name d)
  digitCFG = sprintf('d=''%c'';','0':'9');
  positiveTest(numel(digitCFG)==6*10);
  i = '0123456789';
  g = ['g=dg;g=;' digitCFG];
  o = G.run(i, g);
  positiveTest(isempty(o));                      % every digit accepted
  negativeTest('x', g);                          % non-digit fails

  % test lowers   (defines phrase name l)
  lowerCFG = sprintf('l=''%c'';','a':'z');
  positiveTest(numel(lowerCFG)==6*26);
  i = 'abcdefghijklmnopqrstuvwxyz';
  g = ['g=lg;g=;' lowerCFG];
  o = G.run(i, g);
  positiveTest(isempty(o));                      % every lower accepted
  negativeTest('L', g);                          % non-lower fails
  
  % test uppers   (defines phrase name u)
  upperCFG = sprintf('l=''%c'';','A':'Z');
  positiveTest(numel(upperCFG)==6*26);
  i = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
  g = ['g=lg;g=;' upperCFG];
  o = G.run(i, g);
  positiveTest(isempty(o));                      % every upper accepted
  negativeTest('a', g);                          % non-upper fails
  
  % test ascii   (defines phrase name a)
  asciiCFG = sprintf('a=''%c'';',char(1:126));
  positiveTest(numel(asciiCFG)==6*126);
  i = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
  i = [i 'abcdefghijklmnopqrstuvwxyz'];
  i = [i '~`!@#$%^&*()_-+={[}]|\:;"'',>.?/'];
  g = ['g=ag;g=;' asciiCFG];
  o = G.run(i, g);
  positiveTest(isempty(o));                      % every ascii accepted


  % test digits   (defines phrase name D)
  digitIOG = sprintf('D=''%c''"%c";',repeat('0':'9',2));
  positiveTest(numel(digitIOG)==9*10);
  i = '0123456789';
  g = ['g=Dg;g=;' digitIOG];
  o = G.run(i, g);
  positiveTest(strcmp(o, i));                    % every digit passed
  negativeTest('x', g);                          % non-digit fails
 
  % lower case letters  (defines phrase name L)
  lowerIOG = sprintf('L=''%c''"%c";',repeat('a':'z',2));
  positiveTest(numel(lowerIOG)==9*26);
  i = 'abcdefghijklmnopqrstuvwxyz';
  g = ['g=Lg;g=;' lowerIOG];
  o = G.run(i, g);
  positiveTest(strcmp(o, i));                    % ever lower passed
  negativeTest('L', g);                          % non-lower fails
  
  % upper case letters  (defines phrase name L)
  upperIOG = sprintf('L=''%c''"%c";',repeat('A':'Z',2));
  positiveTest(numel(upperIOG)==9*26);
  i = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
  g = ['g=Lg;g=;' upperIOG];
  o = G.run(i, g);
  positiveTest(strcmp(o, i));                    % every upper passed
  negativeTest('a', g);                          % non-upper fails
  
  % ascii  (defines phrase name A)
  asciiIOG = sprintf('A=''%c''"%c";', repeat(char(1:126),2));
  positiveTest(numel(asciiIOG)==9*126);
  i = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
  i = [i 'abcdefghijklmnopqrstuvwxyz'];
  i = [i '~`!@#$%^&*()_-+={[}]|\:;"'',>.?/'];
  g = ['g=Ag;g=;' asciiIOG];
  o = G.run(i, g);
  positiveTest(strcmp(o, i));                    % every ascii passed
  
  fprintf('gem0: passed %d tests out of %d\n', passed, tests);
  
  %return;
  
  function positiveTest(bexpr)
    tests = tests + 1;
    if bexpr; passed = passed + 1; end
    %tests, passed
  end

  function negativeTest(i, g)  
    try
      res = G.run(i, g);
      positiveTest(false);                       % should not get here
    catch err
      positiveTest(true);                        % should get here
    end
  end

end

