% FILE:     testGem1.m 
% PURPOSE:  test gem (the grammar executing machine)
% METHOD:   test a few grammars -- try bad inputs  -- no SEGV
% REQUIRES: gem0.m iog0.c iog0.h
% EXAMPLE:    
%   testGem0() 

% COPYRIGHT: 2009 W. M. McKeeman.  See license.txt.

function testGem1()  
  EOL    = char(10);                              % ascii code for newline
  tests  = 0;                                     % none yet
  passed = 0;                                     % none yet
  
  G = gem1();                                     % instantiate GEM object
  
  %       -------------- xxxCFG and xxxIOG tests ------------
  
  % --- test G.digitCFG ---
  positiveTest(numel(G.digitCFG)==6*10);
  idata = '0':'9';
  for ch = idata
    positiveTest(isempty(G.run(ch,G.digitCFG)));
  end
  negativeTest('x', G.digitCFG);
  negativeTest('X', G.digitCFG);
  negativeTest(';', G.digitCFG);

  
  % --- test G.lowerCFG ---
  positiveTest(numel(G.lowerCFG)==6*26);
  idata = 'a':'z';
  for ch = idata
    positiveTest(isempty(G.run(ch,G.lowerCFG)));
  end
  negativeTest('7', G.lowerCFG);
  negativeTest('X', G.lowerCFG);
  negativeTest(';', G.lowerCFG);
  
  
  % --- test G.upperCFG ---
  positiveTest(numel(G.upperCFG)==6*26);
  idata = 'A':'Z';
  for ch = idata
    positiveTest(isempty(G.run(ch,G.upperCFG)));
  end
  negativeTest('7', G.upperCFG);
  negativeTest('x', G.upperCFG);
  negativeTest(';', G.upperCFG);

  
  % --- test G.asciiCFG ---
  positiveTest(numel(G.asciiCFG)==126*6);
  idata = char(1:126);
  for ch = idata
    positiveTest(isempty(G.run(ch,G.asciiCFG)));
  end
  
  
  % --- test G.digitIOG ---
  positiveTest(numel(G.digitIOG)==9*10);
  idata = '0':'9';
  for ch = idata
    positiveTest(strcmp(G.run(ch,G.digitIOG),ch));
  end
  negativeTest('x', G.digitIOG);
  negativeTest('X', G.digitIOG);
  negativeTest(';', G.digitIOG);

    
  % --- test G.lowerIOG ---
  positiveTest(numel(G.lowerIOG)==9*26);
  idata = 'a':'z';
  for ch = idata
    positiveTest(strcmp(G.run(ch,G.lowerIOG),ch));
  end
  negativeTest('7', G.lowerIOG);
  negativeTest('X', G.lowerIOG);
  negativeTest(';', G.lowerIOG);
  
  
  % --- test G.upperIOG ---
  positiveTest(numel(G.upperIOG)==9*26);
  idata = 'A':'Z';
  for ch = idata
    positiveTest(strcmp(G.run(ch,G.upperIOG),ch));
  end
  negativeTest('7', G.upperIOG);
  negativeTest('x', G.upperIOG);
  negativeTest(';', G.upperIOG);

  
  % --- test G.asciiIOG ---
  positiveTest(numel(G.asciiIOG)==126*9);
  idata = char(1:126);
  for ch = idata
    positiveTest(strcmp(G.run(ch,G.asciiIOG), ch));
  end
  

  %   -------------- deblanking and scan() ------------
  
  % supress blanks in arbitrary string

  scan=@(txt)G.run(txt,G.nowhite);

  positiveTest(strcmp(scan('x y z'),'xyz'));
  positiveTest(strcmp(scan('; c .  . ('),';c..('));
  positiveTest(strcmp(scan(['x' EOL 'y']),'xy'));
  positiveTest(strcmp(scan(['x''' EOL '''y']),['x''' EOL '''y']));
  positiveTest(strcmp(scan('x'' ''y'),'x'' ''y'));
  positiveTest(strcmp(scan(''''''''), ''''''''));
  

  %     ---------- pretty printing and pretty() -----------
  
  % pretty print a typical grammar
  t1 = G.run('g=ab;a=''A'';b="B";', scan(G.pretty));  % a typical grammar 
  res = ['g = a b;' EOL 'a = ''A'';' EOL 'b = "B";' EOL];
  positiveTest(strcmp(t1, res));
  
  % define pretty printing function
  pretty = @(g)G.run(scan(g), scan(G.pretty));
  
  % pretty on pretty
  t1 = pretty(G.pretty);                       % pretty pretty
  res = G.pretty;
  positiveTest(strcmp(t1(1:120), res(1:120)));

  %            ------------- inverting ---------------
  % invert invert twice gets you back to invert
  i0 = scan(G.invert);
  i1 = G.run(i0, i0);                          % inverted invert
  i2 = G.run(i1, i1);                          % inverted^2 invert
  positiveTest(strcmp(i2, i0));
  
  % right recursive sum and invert
  t1 = G.run('x+x-x', scan(G.sum));            % sum
  res = '4443210';
  positiveTest(strcmp(t1, res));
  g2 = G.run(scan(G.sum), scan(G.invert));     % inverted sum
  t2 = G.run(res,g2);                          % recreated sum
  positiveTest(strcmp(t2,'x+x-x'));
  
  %         ------------ malformed grammar tests --------------                          
  negativeTest('a', 'g');                      % incomplete rule
  negativeTest('b', 'g=');
  negativeTest('c', 'g=''');                
  negativeTest('d', 'g=''x''');        
  negativeTest('e', 'g=";');       
  positiveTest(isempty(G.run('', 'g=;')));
  negativeTest('f', '1');                      % no alpha goal
  negativeTest('g', '1=');
  negativeTest('h', ';');
  negativeTest('i', ';=');
  negativeTest('j', 'g=s;0=;');                % not alpha phrase name
  negativeTest('k', 'g=s;==;s=''x'';');
  negativeTest('l', 'g=s;;=;s=''x''');
  negativeTest('m', 'g=s;s=''x;');             % missing ' in parse
  negativeTest('n', 'g=s;s=''x''"y;');         % missing " in parse
  negativeTest('o', 'g=s;t=''y;s=''x'';');     % missing ' in search
  negativeTest('p', 'g=s;t="y;s=''x'';');      % missing " in search
  negativeTest('q', 'g=s;s''x'';');            % missing = in search
  negativeTest('r', 'g=s;t=''x'';');           % unparsable input
  negativeTest('s', 'g=s;s="x";');             
  negativeTest('t', 'g=s;s=''x''''y'';');     
  negativeTest('u', 'g=?;');                   % unexpected character(parse) 
  negativeTest('v', 'g=s;t=?;');               % unexpected character(search)
  negativeTest('w', 'g=g;');                   % exceed parse stack
  negativeTest('x', 'g=pg;g=;p=;');            % exceed parse stack
  negativeTest('y', 'g=pg;g=;p="1""2";');      % exceed output buffer
  igram = repmat('a',1,100000);                
  igram = ['g=',igram];
  igram = [igram,';a=;'];
  negativeTest('', igram);                     % input grammar too long
  negativeTest(igram, 'r=;')                   % input too long
  idata = char(zeros(1,100)+double('1'));
  g = 'g=pg;g=;p=''1'';';
  positiveTest(isempty(G.run(idata,g)));
  idata = char(zeros(1,100000)+double('1'));   
  negativeTest(idata, g);                      % input text too long
  
  fprintf('gem1: passed %d tests out of %d\n', passed, tests);
  
  %return;
  
  function positiveTest(bexpr)
    tests = tests + 1;
    if bexpr; passed = passed + 1; end
    %tests, passed
  end

  function negativeTest(i, g)  
    try
      res = G.run(i, g);
      positiveTest(false);    % should not get here
    catch err
      positiveTest(true);     % should get here
      %fprintf('gem test %d, %s', tests, err.message);
    end
  end

  
end

