% FILE:     testGem3.m 
% PURPOSE:  test gem (with dewhiting and builtin char classes)
% METHOD:   test a few grammars
% REQUIRES: gem2.m iog2.c
% EXAMPLE:    
%   testGem2() % for all tests

% COPYRIGHT: 2009 W. M. McKeeman.  See license.txt.

function testGem3()  
  EOL    = char(10);                              % ascii code for newline
  tests  = 0;                                     % none yet
  passed = 0;                                     % none yet
  
  G = gem3();                                     % instantiate GEM object

  %       -------------- xxxCFG and xxxIOG tests ------------
  
  % --- test digitCFG ---

  idata = '0123456789';
  for ch = idata
    positiveTest(isempty(G.run(ch,'r=d;','d')));
  end
  negativeTest('x','r=d;','d');  % x is not a digit
  negativeTest('X','r=d;','d');  % X is not a digit
  negativeTest(':','r=d;','d');  % : is not a digit
    
  % --- test lowerCFG ---
  
  lodata = 'a':'z';
  for ch = lodata
    positiveTest(isempty(G.run(ch,'r=l;', 'l')));
  end
  negativeTest('X','r=l;', 'l');  % X is not lower
  negativeTest('1','r=l;', 'l');  % 1 is not lower
  negativeTest(',','r=l;', 'l');  % , is not lower

  % --- test upperCFG ---
  hidata = 'A':'Z';
  for ch = hidata
    positiveTest(isempty(G.run(ch,'r=l;', 'u')));
  end
  negativeTest('y','r=l;', 'u');  % y is not upper
  negativeTest('9','r=l;', 'u');  % 9 is not upper
  negativeTest('+','r=l;', 'u');  % + is not upper
  
  % --- test asciiCFG ---
  
  positiveTest(isempty(G.run('a','r=a;', 'a')));
  positiveTest(isempty(G.run('A','r=a;', 'a')));
  positiveTest(isempty(G.run(';','r=a;', 'a')));
  positiveTest(isempty(G.run('+','r=a;', 'a')));
  positiveTest(isempty(G.run(' ','r=a;', 'a')));
  positiveTest(isempty(G.run(char(EOL),'r=a;', 'a')));
  
  % --- test digitIOG ---
  idata = '0123456789';
  odata = '';
  for ch = idata
    rch = G.run(ch,'r=D;', 'D');
    positiveTest(true);   % did not crash
    odata = [odata rch];  %#ok<AGROW>
  end
  positiveTest(strcmp(odata,idata));
  negativeTest('X','r=D;', 'D');  % X is not a digit

  % --- test lowerIOG ---
  idata = 'a':'z';
  odata = '';
  for ch = idata
    rch = G.run(ch, 'r=L;', 'L');
    positiveTest(true);   % did not crash
    odata = [odata rch];  %#ok<AGROW>
  end
  positiveTest(strcmp(odata,idata));
  
  % --- test upperIOG ---
  idata = 'A':'Z';
  odata = '';
  for ch = idata
    rch = G.run(ch, 'r=L;', 'U');
    positiveTest(true);   % did not crash
    odata = [odata rch];  %#ok<AGROW>
  end
  positiveTest(strcmp(odata,idata));

  
  % --- test asciiIOG ---
  idata = char(1):'~';
  odata = '';
  for ch = idata
    rch = G.run(ch, 'r=A;', 'A');
    positiveTest(true);  % did not crash
    odata = [odata rch];  %#ok<AGROW>
  end
  positiveTest(strcmp(odata,idata));
  
  %  ------- nomulti ------
  
  gr  = {'g = "Hello World";', '    g=  "Hello World"  ;  '};
  for i=1:numel(gr)
    nm  = G.run(gr{i}, G.nomulti, 'A');
    res = G.run('', nm);
    positiveTest(strcmp('Hello World', res));
  end
  
  gr  = {'g = ''Hello World'';', ...
         [EOL EOL 'g' EOL '=' EOL '''Hello World'''   ';' EOL]};
  for i=1:numel(gr)
    nm  = G.run(gr{i}, G.nomulti, 'A');
    res = G.run('Hello World', nm);
    positiveTest(isempty(res));
  end
  
  %     ---------- pretty printing and pretty() -----------
  
  % pretty print a typical grammar
  t1 = G.run('g=ab;a=''A'';b="B";', G.pretty, 'UAL');
  res = ['g = a b;' EOL 'a = ''A'';' EOL 'b = "B";' EOL];
  positiveTest(strcmp(t1, res));
  
  % pretty on pretty
  t1 = G.run(G.pretty, G.pretty, 'UAL');      % pretty pretty
  positiveTest(strcmp(t1, G.pretty));

  %            ------------- inverting ---------------
  % invert invert twice gets you back to invert
  i0 = G.invert;
  i1 = G.run(i0, i0, 'A');                     % inverted invert
  i2 = G.run(i1, i1, 'A');                     % inverted^2 invert
  positiveTest(strcmp(i2, i0));
    
  %        -------- arithmetic expressions --------------
  
  % convert infix to postfix
  t1 = G.run('x*(y+3)-x/7', G.postfix, 'LD');
  res = 'xy3+*x7/-';
  positiveTest(strcmp(t1, res));
%{  
  % pretty on kleene
  tests = tests + 1;
  t1 = G.run(G.scan(G.kleene),G.scan(G.pretty), 'LUA');
  res = G.scan(G.kleene);
  if strcmp(G.scan(t1), res); passed = passed+1; end;

  % remove * from simple CFG
  tests = tests + 1;
  t1 = G.run('r=a*b*;',G.scan(G.nostar1), 'lua');
  res = 'A=aA;A=;B=bB;B=;';
  if strcmp(G.scan(t1), res); passed = passed+1; end;

  % remove * from kleene
  tests = tests + 1;
  k1 = G.run(G.scan(G.kleene),G.scan(G.nostar1), 'lua');
  passed = passed+1;

  % replace a* with A, b* with B in simple CFG
  tests = tests + 1;
  t1 = G.run('r=a*b*;',G.scan(G.nostar2), 'LUA');
  res = 'r=AB;';
  if strcmp(G.scan(t1), res); passed = passed+1; end;

  % replace r* with R, p* with P in kleene
  tests = tests + 1;
  k2 = G.run(G.scan(G.kleene),G.scan(G.nostar2), 'LUA');
  passed = passed+1;
  
  % run kleene on r=a*b*;
  tests = tests + 1;
  g1 = 'r=a*b*;';
  k3 = G.scan([k2 k1]);
  t1 = G.run(g1,k3, 'lua');
  if isempty(G.scan(t1)); passed = passed+1; end;

  % run kleene on kleene
  tests = tests + 1;
  t1 = G.run(G.scan(G.kleene),k3, 'lua');
  if isempty(G.scan(t1)); passed = passed+1; end;
  traceit();
  %}
  
  fprintf('gem3: passed %d tests out of %d\n', passed, tests);
  
  %return;
  
  function positiveTest(bexpr)
    tests = tests + 1;
    if bexpr; passed = passed + 1; end
    %tests, passed
  end

  function negativeTest(i, g, f)  
    try
      res = G.run(i, g, f);
      positiveTest(false);    % should not get here
    catch err
      positiveTest(true);     % should get here
      %fprintf('gem test %d, %s', tests, err.message);
    end
  end

end

