% FILE:     testGem4.m 
% PURPOSE:  test gem (the grammar executing machine)
% METHOD:   multichar input and output
% REQUIRES: gem.m iog4.c iog4.h
% EXAMPLE:    
%   testGem4() % for all tests

% COPYRIGHT: 2009 W. M. McKeeman.  See license.txt.

function testGem4()  
  EOL = char(10);                                 % ascii newline
  tests  = 0;                                     % none yet
  passed = 0;                                     % none yet
  
  G = gem4();                                     % instantiate GEM object

  positiveTest(strcmp('',     G.run('', 'g=;')));
  positiveTest(strcmp('abc',  G.run('', 'g="abc";')));
  positiveTest(strcmp('Hello World!', G.run('', 'g="Hello World!";')));
  positiveTest(strcmp('''',   G.run('', 'g="''";')));
  positiveTest(strcmp('"',    G.run('', 'g=""";')));
  positiveTest(strcmp('',     G.run('"', 'g=''"'';')));
  positiveTest(strcmp('',     G.run('''', 'g='''''';')));
  positiveTest(strcmp('',     G.run('ab', 'g=''ab'';')));
  positiveTest(strcmp('',     G.run('''x', 'g='''''' ''x'';')));
  positiveTest(strcmp('',     G.run('X''', 'g=''X'' '''''';')));
  positiveTest(strcmp('abc',  G.run('xyz', 'g=''xyz''"abc";')));
  positiveTest(strcmp('abc',  G.run('xyz', 'g="abc"''xyz'';')));
  positiveTest(strcmp('abc',  G.run('xyz', 'g = "abc" ''xyz'' ; ')));
  positiveTest(strcmp('"',    G.run('xyz', 'g="""''xyz'';')));
  positiveTest(strcmp('',     G.run('1', 'g=a;g=b;a=''1'';b=''2'';')));
  positiveTest(strcmp('',     G.run('2', 'g=a;g=b;a=''1'';b=''2'';')));
  positiveTest(strcmp('O',    G.run('1','g=a;g=b;a="O"''1'';b="T"''2'';')));
  positiveTest(strcmp('O',    G.run('1','g = a;g = b;a = "O"''1'';b="T"''2'';')));
  positiveTest(strcmp('',     G.run('123', 'g=dg;g=;', 'd')));
  positiveTest(strcmp('123',  G.run('123', 'g=Dg;g=;', 'D')));
  positiveTest(strcmp('',     G.run('1', 'g=a;a=b;b=c;c=d;d=f;f=''1'';')));  
  pin  = 'x =  y ;  y   = ''a'' "123"   ;   ';
  pout = ['x = y;' EOL 'y = ''a'' "123";' EOL];
  positiveTest(strcmp(pout, G.run(pin, G.pretty, 'ALU')));
  pin  = 'x =  y  * ;  y   = ''a'' "123"   ;   ';
  pout = ['x = y*;' EOL 'y = ''a'' "123";' EOL];
  positiveTest(strcmp(pout, G.run(pin, G.pretty, 'ALU')));
  pin  = 'x =  y  + ;  y   = ''a'' "123"   ;   ';
  pout = ['x = y+;' EOL 'y = ''a'' "123";' EOL];
  positiveTest(strcmp(pout, G.run(pin, G.pretty, 'ALU')));  
  pin = 'x+3*y-z*(2+x)';
  pout = 'x3y*+z2x+*-';
  positiveTest(strcmp(pout, G.run(pin, G.postfix, 'LD')));
  positiveTest(strcmp(['g =;' EOL], G.run('g=              ;', G.pretty, 'ULA')));
  pin = 'g = p+; p = t*; t = a;';
  pout = 'g=pp*;p=t*;t=a;';
  positiveTest(strcmp(pout, G.run(G.scan(pin), G.noplus, 'ALU'))); 
  negativeTest('',    'g=g;', '');     % left recursion
  negativeTest('',    'g=rg;r="a";');  % infinite output
  negativeTest('a',   'g=''b'';');     % bad input
    
  fprintf('gem4: passed %d tests out of %d\n', passed, tests);
  
  %return;
  
  function positiveTest(bexpr)
    tests = tests + 1;
    if bexpr; passed = passed + 1; end
    %tests, passed
  end

  function negativeTest(i, g, f)  
    try
      res = G.run(i, g, f);
      positiveTest(false);    % should not get here
    catch err
      positiveTest(true);     % should get here
      %fprintf('gem test %d, %s', tests, err.message);
    end
  end

end

